package main;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.JsonNode;
import com.mashape.unirest.http.Unirest;
import com.mashape.unirest.http.exceptions.UnirestException;
import elements.Chat;
import elements.Persistence;
import elements.exceptions.PoleBlockedException;
import elements.exceptions.ReadErrorException;
import elements.exceptions.SaveErrorException;
import org.json.JSONArray;
import org.json.JSONObject;

import java.time.LocalDate;
import java.util.*;

/**
 * Created by vylion on 4/1/16.
 */
public class Vylbot {
    private static final String TOKEN = "165232232:AAGVoVm1AA_cP2RNGh3sR4nPX9hQvujr_ls";
    private static final String LOGTAG = "VYLBOT";
    private static final String USERNAME = "VylionBot";
    private static final String BASE_URL = "https://api.telegram.org/bot" + TOKEN;
    private static final String PARSE_MARKDOWN = "Markdown";
    private static final String PARSE_HTML = "HTML";

    private static final int WARNING_COUNT_START = 0;
    private static final int START_MESSAGE = WARNING_COUNT_START;
    private static final int HELP_MESSAGE = START_MESSAGE + 1;
    private static final int WARNING_UNKNOWN_ERROR = HELP_MESSAGE + 1;
    private static final int WARNING_NO_USERNAME = WARNING_UNKNOWN_ERROR + 1;
    private static final int WARNING_POLE_BLOCKED = WARNING_NO_USERNAME + 1;
    private static final int WARNING_NOT_IMPLEMENTED = WARNING_POLE_BLOCKED + 1;
    private static final int WARNING_NO_GROUP = WARNING_NOT_IMPLEMENTED + 1;
    private static final int WARNING_COUNT_END = WARNING_NO_GROUP + 1;

    private List<String> commands;
    private Map<Long, Chat> chats;

    public Vylbot() throws ReadErrorException {
        //chats = Persistence.getInstance().readChats();
    }

    public String getUsername() {
        return USERNAME;
    }

    public String getToken() {
        return TOKEN;
    }

    public String myName() {
        return "Vylionbot";
    }

    //--- Http handling

    public HttpResponse<JsonNode> sendMessage(Long chatId, String text) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(Long chatId, String text, String parseMode) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .field("parse_mode", parseMode)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(String channel, String text) throws UnirestException {
        System.out.println("Sending message to " + channel + ": \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", channel)
                .field("text", text)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(String channel, String text, String parseMode) throws UnirestException {
        System.out.println("Sending message to " + channel + ": \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", channel)
                .field("text", text)
                .field("parse_mode", parseMode)
                .asJson();
    }

    public HttpResponse<JsonNode> replyMessage(Long chatId, Integer repliedMessage, String text) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .field("reply_to_message_id", repliedMessage)
                .asJson();
    }

    public HttpResponse<JsonNode> getUpdates(Integer offset) throws UnirestException {
        return Unirest.post(BASE_URL + "/getUpdates")
                .field("offset", offset)
                .asJson();
    }

    public HttpResponse<JsonNode> getUpdates(Integer offset, Integer timeout) throws UnirestException {
        return Unirest.post(BASE_URL + "/getUpdates")
                .field("offset", offset)
                .field("timeout", timeout)
                .asJson();
    }

    public HttpResponse<JsonNode> getPokeGoStatus() throws UnirestException {
        return Unirest.get("http://pokemongosource.com/is-pokemon-go-down/json")
                .asJson();
    }

    //--- bot thinking

    private boolean askPogeGoStatus() throws UnirestException {
        return getPokeGoStatus().getBody().getObject().getBoolean("pokemon-go-up");
    }

    public void run() throws UnirestException {
        int last_upd_id = 0;
        HttpResponse<JsonNode> response;
        System.out.println("Listening.\n");

        while (true) {
            response = getUpdates(last_upd_id++);
            if (response.getStatus() == 200) {
                JSONArray responses = response.getBody().getObject().getJSONArray("result");
                if (responses.isNull(0)) continue;
                else last_upd_id = responses.getJSONObject(responses.length() - 1).getInt("update_id") + 1;

                for (int i = 0; i < responses.length(); i++) {
                    if(responses.getJSONObject(i).has("message")) {
                        JSONObject message = responses.getJSONObject(i).getJSONObject("message");

                        processMessage(message);
                    }
                }
            }
        }
    }

    private void processMessage(JSONObject message) throws UnirestException {
        //Get basic compulsory variables
        long chat_id = message.getJSONObject("chat").getLong("id");
        int message_id = message.getInt("message_id");
        String chatType = message.getJSONObject("chat").getString("type");
        String name = message.getJSONObject("from").getString("first_name");
        boolean isGroup = chatType.contains("group");

        //Declare basic optional variables
        String user = null;
        String chatName = null;

        //Check for user last name
        if (message.getJSONObject("from").has("last_name"))
            name += " " + message.getJSONObject("from").getString("last_name");

        //Check for user username
        if (message.getJSONObject("from").has("username"))
            user = message.getJSONObject("from").getString("username");

        //Get group chat name
        if (message.getJSONObject("chat").has("title"))
            chatName = message.getJSONObject("chat").getString("title");

        //Console log
        boolean full_log = false;
        if (full_log) System.out.println("\n" + message);
        else {
            if (message.getJSONObject("chat").has("title"))
                System.out.println("\nMessage received from chat " +
                        message.getJSONObject("chat").getString("title") +
                        " (chat id: " + chat_id + ")");
            else if (user != null)
                System.out.println("\nMessage received from user @" + user +
                        " (chat id: " + chat_id + ")");
            else
                System.out.println("\nMessage received from user named " + name +
                        " (chat id: " + chat_id + ")");
        }

        //Actually read the message
        if (message.has("text")) {
            String text = message.getString("text");
            handleInput(chat_id, isGroup, chatName, name, user, text);
            return;
        }
    }

    //--- Command handling

    private void handleInput(long chat_id, boolean group, String chat_name, String name, String user, String text) throws UnirestException {
        if(text.startsWith("/start") &&
                (!text.substring("/start".length()).startsWith("@") ||
                        text.substring("/start".length()).startsWith("@" + USERNAME)))
            handleStart(chat_id);

        if(text.startsWith("/help") &&
                (!text.substring("/help".length()).startsWith("@") ||
                        text.substring("/help".length()).startsWith("@" + USERNAME)))
            handleHelp(chat_id);

        if(text.startsWith("/pokego") &&
                (!text.substring("/pokego".length()).startsWith("@") ||
                        text.substring("/pokego".length()).startsWith("@" + USERNAME)))
            handlePokeGoStatus(chat_id);

        if(text.startsWith("/whoami") &&
                (!text.substring("/whoami".length()).startsWith("@") ||
                        text.substring("/whoami".length()).startsWith("@" + USERNAME)))
            handleWhoami(chat_id, group, chat_name, name, user);
    }

    private void handleStart(long chat_id) throws UnirestException {
        sendMessage(chat_id, "Hola. Soy el bot personal de Vylion");
        return;
    }

    private void handleHelp(long chat_id) throws UnirestException {
        String help = "Aquí tienes una lista de comandos que acepto:\n\n" +
                "/start - Repite el saludo inicial\n" +
                "/help - Saca esta lista\n" +
                "/pokego - Muestra el status de los servidores de PokeGo " +
                "(según una web de terceros).\n" +
                "\n_Vylbot v1.2.0_";

        sendMessage(chat_id, help, PARSE_MARKDOWN);
        return;
    }

    private void handlePokeGoStatus(long chat_id) throws UnirestException {
        String status = "Según mis fuentes, ";
        if(askPogeGoStatus())
            status += "Pokémon Go funciona";
        else
            status += "los pokeservidores tienen un pokeproblema";

        sendMessage(chat_id, status);
    }

    private void handleWhoami(long chat_id, boolean group, String chat_name, String user, String name) throws UnirestException {
        String s = "Eres " + name;
        if(user != "blank_username") s+= ", con nombre de usuario @" + user;
        s += ", en el chat";
        if(group && chat_name != null) s+= " " + chat_name;
        s += " de id " + chat_id + "\n\n";
        s += "En el PC que me mantiene, la fecha actual es " + LocalDate.now() +
                " en el formato ISO-8601.";
        sendMessage(chat_id, s);
        return;
    }

    private boolean exists(long chat_id, String user) {
        return  chats.containsKey(chat_id) && chats.get(chat_id).exists(user);
    }

    private void saveChat(long chat_id) throws SaveErrorException {
        try {
            Persistence.getInstance().saveChat(chat_id, chats.get(chat_id));
        } catch (SaveErrorException e) {
            e.printStackTrace();
            throw e;
        }
    }

    private String getWarning(int warning_id) {
        switch (warning_id) {
            case WARNING_UNKNOWN_ERROR:
                return "Algo ha ido MUY mal.";

            case WARNING_NOT_IMPLEMENTED:
                return "Aún no está implementado.";

            case WARNING_NO_GROUP:
                return "Debes estar en un grupo para poder hacer eso.";

            case WARNING_NO_USERNAME:
                return "No tienes nombre de usuario. Regístrate con uno en la " +
                        "configuración de Telegram para poder participar.";

            case WARNING_POLE_BLOCKED:
                return "La pole de hoy ya ha sido conseguida. Vuélvelo a intentar mañana.";

            default: return "Entrada incorrecta.";
        }
    }

    private String getFormatExample(int command) {
        return "Entrada incorrecta.";
    }
}